import { View } from "../../../framework/View";
import { ViewModel, ViewModelOptions } from "../../../framework/ViewModel";
import { ObservableProperty } from "../../../framework/observable";
export type InputType = "text" | "checkbox" | "file" | "integer" | "float" | "range";
export type ValueUpdateMode = "OnChange" | "OnInput";
export interface InputViewModelOptions<TValue> extends ViewModelOptions {
    value?: TValue | ObservableProperty<TValue>;
    inputValidator?: (event: Event, inputViewModel: InputViewModelOptions<TValue>, value: string) => Promise<boolean>;
    placeholder?: string | ObservableProperty<string>;
    valueUpdateMode?: ValueUpdateMode;
    min?: number;
    max?: number;
    step?: number;
    type?: InputType;
}
declare abstract class InputViewModel<TValue> extends ViewModel {
    protected constructor(inputType: InputType, options?: InputViewModelOptions<TValue>);
    readonly value: ObservableProperty<TValue>;
    readonly inputValidator: (event: Event, inputViewModel: InputViewModelOptions<TValue>, value: string) => Promise<boolean>;
    readonly placeholder: ObservableProperty<string>;
    readonly valueUpdateMode: ValueUpdateMode;
    readonly type: InputType;
}
export declare class InputTextViewModel extends InputViewModel<string> {
    constructor(options?: InputViewModelOptions<string>);
}
export declare class InputCheckboxViewModel extends InputViewModel<boolean> {
    constructor(options?: InputViewModelOptions<boolean>);
}
export declare class InputFileViewModel extends InputViewModel<FileList> {
    constructor(options?: InputViewModelOptions<FileList>);
}
export declare class InputIntegerViewModel extends InputViewModel<number> {
    constructor(options?: InputViewModelOptions<number>);
}
export declare class InputFloatViewModel extends InputViewModel<number> {
    constructor(options?: InputViewModelOptions<number>);
}
export declare class InputView<TValue> extends View<InputViewModelOptions<TValue>> {
    constructor(viewModel: InputViewModelOptions<TValue>, designTimeOptions?: InputViewModelOptions<TValue>);
    protected build(): void;
}
export {};
