import * as kendo from "@progress/kendo-ui/js/kendo.grid.js";
import { View, registerView } from "../../../framework/View";
import { ViewModel, extendViewModel } from "../../../framework/ViewModel";
import { unwrap, isObservable, utils } from "knockout";
export class GridViewModel extends ViewModel {
    constructor(options) {
        super(options);
        if (options) {
            this.items = options.items;
            this.idProperty = options.idProperty;
            this.columns = options.columns;
            this.pageSize = options.pageSize;
            this.sortable = options.sortable;
            this.scrollable = options.scrollable;
            this.editable = options.editable;
            this.filterable = options.filterable;
            this.toolbar = options.toolbar;
            this.selectable = options.selectable;
            this.selectedItem = options.selectedItem;
            this.selectedItems = options.selectedItems;
            this.onItemChange = options.onItemChange;
            this.onAdd = options.onAdd;
            this.onRemove = options.onRemove;
        }
    }
    items;
    idProperty;
    columns;
    pageSize;
    sortable;
    scrollable;
    editable;
    filterable;
    toolbar;
    selectable;
    selectedItem;
    selectedItems;
    onItemChange;
    onAdd;
    onRemove;
    selectItem(index = null) {
        if (!index) {
            index = 0;
        }
        const items = unwrap(this.items);
        const item = items[index];
        if (item) {
            if (this.selectedItem) {
                const observableProperty = this.selectedItem;
                if (observableProperty.set) {
                    observableProperty.set(item);
                }
            }
            if (this.selectedItems) {
                const observableArrayProperty = this.selectedItems;
                if (observableArrayProperty.set) {
                    observableArrayProperty.set([item]);
                }
            }
        }
    }
}
export class GridView extends View {
    constructor(viewModel, designTimeOptions) {
        super({
            tagName: "div",
            viewModel: viewModel
        }, extendViewModel({
            viewType: "KendoGrid"
        }, designTimeOptions));
    }
    grid;
    dataSource;
    getItemById(id) {
        if (this.viewModel.idProperty) {
            const items = this.dataSource.data();
            return utils.arrayFirst(items, (item) => { return item[this.viewModel.idProperty] === id; });
        }
        throw "Can only getItem when idProperty is set";
    }
    build() {
        super.build();
        const viewModel = this.viewModel;
        this.dataSource = new kendo.data.HierarchicalDataSource({
            data: unwrap(viewModel.items),
            pageSize: this.viewModel.pageSize || 30,
            change: (event) => {
                var items = event.items;
                switch (event.action) {
                    case "itemchange":
                        if (viewModel.onItemChange) {
                            viewModel.onItemChange(items, event.sender);
                        }
                        break;
                    case "add":
                        if (viewModel.onAdd) {
                            viewModel.onAdd(items, event.sender);
                        }
                        break;
                    case "remove":
                        if (viewModel.onRemove) {
                            viewModel.onRemove(items, event.sender);
                        }
                        break;
                }
            }
        });
        let changeTriggeredByGrid = false;
        this.grid = new kendo.ui.Grid(this.element, {
            dataSource: this.dataSource,
            columns: this.viewModel.columns,
            toolbar: this.viewModel.toolbar,
            scrollable: this.viewModel.scrollable,
            sortable: this.viewModel.sortable,
            editable: this.viewModel.editable,
            filterable: this.viewModel.filterable,
            change: (event) => {
                changeTriggeredByGrid = true;
                if (this.viewModel.selectedItem) {
                    const selectedItem = getSelectedDataItem(event.sender);
                    const observableProperty = this.viewModel.selectedItem;
                    if (observableProperty.set) {
                        observableProperty.set(selectedItem);
                    }
                }
                if (this.viewModel.selectedItems) {
                    const selectedItems = getSelectedDataItems(event.sender);
                    const observableArrayProperty = this.viewModel.selectedItems;
                    if (observableArrayProperty.set) {
                        observableArrayProperty.set(selectedItems);
                    }
                }
            },
            selectable: this.viewModel.selectable
        });
        if (isObservable(viewModel.items)) {
            viewModel.items.subscribe((items) => {
                if (items) {
                    this.dataSource.data(new kendo.data.ObservableArray(items));
                }
                else {
                    this.dataSource.data([]);
                }
            });
        }
        if (isObservable(viewModel.selectedItem)) {
            viewModel.selectedItem.subscribe((selectedItem) => {
                if (changeTriggeredByGrid) {
                    changeTriggeredByGrid = false;
                    return;
                }
                if (selectedItem && viewModel.idProperty) {
                    setTimeout(() => {
                        const id = selectedItem[viewModel.idProperty];
                        const item = this.getItemById(id);
                        const row = this.grid.tbody.find("tr[data-uid='" + item.uid + "']");
                        if (row) {
                            this.grid.select(row);
                        }
                    }, 10);
                }
                else {
                    this.grid.select();
                }
            });
        }
        if (isObservable(viewModel.selectedItems)) {
            viewModel.selectedItems.subscribe((selectedItems) => {
                if (changeTriggeredByGrid) {
                    changeTriggeredByGrid = false;
                    return;
                }
                if (selectedItems && selectedItems.length > 0 && viewModel.idProperty) {
                    setTimeout(() => {
                        const rows = new Array();
                        for (let index = 0; index < selectedItems.length; index++) {
                            const selectedItem = selectedItems[index];
                            const id = selectedItem[viewModel.idProperty];
                            const item = this.getItemById(id);
                            const row = this.grid.tbody.find("tr[data-uid='" + item.uid + "']");
                            rows.push(row);
                        }
                        this.grid.select(...rows);
                    }, 10);
                }
                else {
                    this.grid.select();
                }
            });
        }
        setTimeout(() => {
            this.grid.resize(true);
        }, 5);
    }
}
export function getSelectedDataItem(grid) {
    const selection = grid.select();
    if (selection.length === 1) {
        return grid.dataItem(selection[0]).toJSON();
    }
    return null;
}
export function getSelectedDataItems(grid) {
    const selection = grid.select();
    const selectedDataItems = new Array();
    if (selection && selection.length > 0) {
        for (let index = 0; index < selection.length; index++) {
            selectedDataItems.push(grid.dataItem(selection[index]).toJSON());
        }
    }
    return selectedDataItems;
}
registerView(GridViewModel, GridView);
//# sourceMappingURL=Grid.js.map