import * as kendo from "@progress/kendo-ui/js/kendo.menu.js";
import { isObservable, unwrap } from "knockout";
import { View, render } from "../../../framework/View";
import { ViewModel, applyViewModelToElement, extendViewModel } from "../../../framework/ViewModel";
const getContent = "getContent";
export class MenuView extends View {
    constructor(viewModel, designTimeOptions) {
        super({
            tagName: "div",
            viewModel: viewModel
        }, extendViewModel({
            classNames: ["Menu"],
            viewType: "KendoMenu"
        }, designTimeOptions));
    }
    menu;
    dataSource;
    build() {
        super.build();
        const viewModel = this.viewModel;
        this.dataSource = new kendo.data.HierarchicalDataSource({});
        this.menu = new kendo.ui.Menu(this.element, {
            dataTextField: "text",
            dataSource: this.dataSource
        });
        this.menu.bind("dataBound", (event) => {
            event.sender.wrapper.find(".k-menu-item[postrender]").each((index, element) => {
                const uid = element.getAttribute("data-uid");
                const hierarchicalListItemViewModel = this.dataSource.getByUid(uid);
                if (hierarchicalListItemViewModel) {
                    applyViewModelToElement(hierarchicalListItemViewModel, element);
                    if (hierarchicalListItemViewModel[getContent]) {
                        const content = hierarchicalListItemViewModel[getContent]();
                        if (content instanceof ViewModel) {
                            const firstChild = element.firstElementChild;
                            firstChild.style.padding = "0";
                            firstChild.innerHTML = "";
                            firstChild.appendChild(render(content));
                        }
                    }
                }
                element.removeAttribute("postrender");
            });
        });
        this.buildItems(unwrap(viewModel.items));
        if (isObservable(viewModel.items)) {
            viewModel.items.subscribe((items) => {
                this.buildItems(items);
            });
        }
    }
    buildItems(listItems) {
        const itemDictionary = {};
        const topLevelItems = new Array();
        if (listItems) {
            listItems.forEach((listItem) => {
                listItem["attr"] = { postrender: true };
                if (typeof listItem.text !== "string") {
                    var content = listItem.text;
                    listItem[getContent] = () => { return content; };
                    listItem.text = "<post>";
                }
                if (unwrap(listItem.visible) !== false) {
                    itemDictionary[listItem.id] = listItem;
                }
            });
            Object.keys(itemDictionary).forEach((key) => {
                const menuItem = itemDictionary[key];
                if (menuItem.parentId) {
                    const parent = itemDictionary[menuItem.parentId];
                    if (parent) {
                        if (!parent.items) {
                            parent.items = [];
                        }
                        parent.items.push(itemDictionary[menuItem.id]);
                    }
                }
                else {
                    topLevelItems.push(itemDictionary[menuItem.id]);
                }
            });
        }
        this.dataSource.data(topLevelItems);
    }
}
//# sourceMappingURL=Menu.js.map