import { getGuid } from "../common/common";
var tokenProvider;
export function init(authorizationTokenProvider) {
    tokenProvider = authorizationTokenProvider;
}
export async function get(input, authorized, responseType, init) {
    if (!init) {
        init = {};
    }
    await setGetInitValues(init, authorized);
    const response = await fetch(input, init);
    return await getResponseValue(response, responseType);
}
export async function post(input, data, authorized, responseType, init) {
    if (!init) {
        init = {};
    }
    await setPostInitValues(init, authorized, data);
    const response = await fetch(input, init);
    return await getResponseValue(response, responseType);
}
function getFilename(response) {
    const contentDispositionHeader = response.headers.get("Content-Disposition");
    if (contentDispositionHeader) {
        const parts = contentDispositionHeader.split(";");
        for (let part of parts) {
            if (part.indexOf("filename=") > -1) {
                return part.replace("filename=", "").trim();
            }
        }
    }
    return null;
}
async function getResponseValue(response, responseType) {
    if (response.ok) {
        switch (responseType) {
            case "void":
                return null;
            case "json":
                return response.json();
            case "blob":
                return {
                    blob: await response.blob(),
                    name: getFilename(response)
                };
            case "arrayBuffer":
                return response.arrayBuffer();
            default:
                throw new Error("The responseType" + responseType + " is not valid");
        }
    }
    const errorJson = await response.json();
    throw new Error(errorJson.message);
}
async function setGetInitValues(init, authorized) {
    await setHeaders(init, authorized);
}
async function setPostInitValues(init, authorized, data) {
    await setHeaders(init, authorized);
    if (!init.method) {
        init.method = "POST";
    }
    if (!init.mode) {
        init.mode = "cors";
    }
    if (!init.cache) {
        init.cache = "no-cache";
    }
    if (!init.credentials) {
        init.credentials = "same-origin";
    }
    if (!init.headers) {
        init.headers = {};
    }
    if (!init.headers["Content-Type"] && !(data instanceof FormData)) {
        init.headers["Content-Type"] = "application/json";
    }
    if (data instanceof FormData) {
        delete init.headers["Content-Type"];
    }
    if (!init.redirect) {
        init.redirect = "follow";
    }
    if (!init.referrerPolicy) {
        init.referrerPolicy = "no-referrer";
    }
    if (!init.body) {
        if (data instanceof FormData) {
            init.body = data;
        }
        else {
            init.body = JSON.stringify(data);
        }
    }
}
async function setHeaders(init, authorized) {
    if (!init.headers) {
        init.headers = {};
    }
    init.headers["X-Request-ID"] = getGuid();
    if (authorized) {
        if (tokenProvider) {
            init.headers["Authorization"] = tokenProvider();
        }
        else {
            throw "Token provider not set";
        }
    }
}
//# sourceMappingURL=fetch.js.map